//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public License
// along with this program.  If not, see http://www.gnu.org/licenses/.
//
// @author Brian Ricks
//

#ifndef RADIOPACKETGENERATOR_H_
#define RADIOPACKETGENERATOR_H_

#include "inet/physicallayer/contract/packetlevel/IRadio.h"

// Generates packets for direct transmission by a radio.  No MAC is assumed.
// Useful for simulation of jammers.
class RadioPacketGenerator : public cSimpleModule, public cListener {
public:
    RadioPacketGenerator();
    virtual ~RadioPacketGenerator();

    // @override from cListener
    virtual void receiveSignal(cComponent *source, simsignal_t signalID, long val, cObject *details);

protected:
    virtual void initialize(int stage);
    virtual int numInitStages() const override { return inet::NUM_INIT_STAGES; }
    virtual void handleMessage(cMessage *msg);
    virtual void finish();

private:
    inet::physicallayer::IRadio::TransmissionState txState;  // the current radio transmission state
    inet::physicallayer::IRadio::RadioMode txRadioMode;  // the current radio mode
    inet::physicallayer::IRadio *txRadio;  // pointer to the radio which we use for transmission
    cMessage *scheduleMsg;

    long generatedPktLength;  // in bytes
    simtime_t sendDelay; // delay between packet sending, in seconds

    bool isScheduled; // true if a packet is currently scheduled

    void sendPacket();
    void attemptNextPacketTransmission();
};

#endif /* RADIOPACKETGENERATOR_H_ */
